#!/usr/bin/python -u

import numpy as np
from time import time

from susmost import make_single_point_zmatrix, make_square_plane_cell, \
	normal_lattice_task, mc, savexyz, nearest_int, SiteStateType, register_property, sitestate
from susmost.mc import make_regr, metropolis, tmatrix
from susmost.savexyz import save_cells_as_xyz

import pickle

#constants
INF_E  = 1E6 # prohibitive energy level
k_B = 8.3144598E-3 # kJ/(mol*K)
#k_B = 0.000086	# eV
#k_B = 1.0
a = 1.0 # lattice parameter
r_cut_off= 8.0 + .0001
precission = 6

#parameters
L = 36			#linear size of the lattice in a
T = 100.0		#temperature in K
T_list = [T]	#list of temperatures for parallel tempering

#types of interaction and its energies in kJ/mol
mol_int = {('molecule_A','molecule_B'):0.0,('molecule_A','molecule_A'):4.0,('molecule_B','molecule_B'):6.0}

#interaction function that takes into account types of the molecules and
#arangement of the adsorption sites on the lattice
interaction_function = lambda cc1,cc2: nearest_int(cc1,cc2,dop_int=mol_int)

#type of the lattice
cell,atoms = make_square_plane_cell(a)

#properties of interest
register_property('coverage_A','==', None, sitestate.join_cover)
register_property('coverage_B','==', None, sitestate.join_cover)


def binary_gas_MC_simulation(muA, muB):
	res_calc = [] # for each element in muA_list

	# Initialize states of the lattice site
	molA_state = SiteStateType('molecule_A', make_single_point_zmatrix(), muA, ['N'],
				coverage=1., coverage_A=1., coverage_B=0.)
	molB_state = SiteStateType('molecule_B', make_single_point_zmatrix(), 0.0, ['O'],
				coverage=1., coverage_A=0., coverage_B=1.)
	empty_state = SiteStateType('Empty',make_single_point_zmatrix(), 0.0, ['H'],
				coverage=0., coverage_A=0., coverage_B=0.)

	states, edges_dict, IM = normal_lattice_task(cell, atoms, interaction_function, r_cut_off, INF_E, precission, [molA_state, molB_state, empty_state], max_dimensions=[-1,-1,0])

	regr,pbcell = make_regr(edges_dict, cell, [ ('Pt', [0.,0.,0.] )], [L,L,1], [True, True, False], r_cut_off, precission)

	# key constants:
	cells_count	=	len(regr)
	edges_count	=	len(edges_dict)
	lc_count 	= 	len(states)
	#----------------------------------

	cells = np.zeros(cells_count, dtype=int)	# fill by 0-th
	allowed_cells = np.full((cells_count,lc_count), True, dtype=bool)	# all allowed
	transitions = tmatrix.make_dumb_tmatrix(states)
	internal_energies =  [[E for E,_ in s.EP_list()] for s in states]	# from adsorption energies, s.E - is a  list of energies
	lattice_graph=regr.as_nparray(edges_count) # fill lattice_graph from regr
	interaction = IM.asarray()  # fill interaction from IM

	additive_params = [[[props['coverage'].value, props['coverage_A'].value, props['coverage_B'].value] for _,props in s.EP_list()] for s in states]

	p_dict = {"coverage":0, "coverage_A":1, "coverage_B":2, "T_index":-1, "E_per_site":-2, "E_int_per_site":-3, "accept_rate":-4, "free_E_incell_per_site":-5, \
	"minE_incell_per_site":-6, "S_incell_per_site":-7, "avgE_incell_per_site":-8}

	m = mc.Metropolis(lattice_graph, cells, allowed_cells, interaction, internal_energies, transitions, T_list, additive_params, k_B, int(time()))
	m.phys_diffusion = False
	m.E_inf = INF_E # Max energy
	m.PT_period = 100 # Each m.PT_period steps we try to switch the temperature
	m.diff_cui_min = 1 # Minimal (exchange the state of a single site)
	m.diff_cui_max = 2 # Maximal (exchange the states of two sites)

	# Save a .xyz state of the lattice
	xyz_fn = "muA_" + str(muA) + ".xyz"
	def callback(m):
		save_cells_as_xyz(xyz_fn, regr, cells, states, pbcell)

	#	log_periods_cnt - Amount of points for observation
	#	log_period_steps - Amount of trial steps between observations
	#	Total amount of observation to log_period_steps * log_periods_cnt

	t0 = time()
	print "START MC"
	stat_log = mc.run(m, log_periods_cnt = 200, log_period_steps = 1000, log_callback=callback, params_period_steps = 1000, relaxation_steps = 10000000)
	print "FINISH MC", time()-t0

	for T, param_log in zip(T_list, m.full_params_log):
		stat = np.mean(param_log, axis=0)[[p_dict["coverage"],p_dict["coverage_A"],p_dict["coverage_B"],p_dict["E_per_site"]]] # cov,cov_A,cov_B,E
		res_i = [T, muA, muB, stat[0], stat[1], stat[2], stat[3]]
		res_calc.append(res_i)
		print 'res_i {} {} {} {}'.format(*res_i)

	return res_calc

if __name__ == '__main__':

	res = []

	for muB in np.arange(0., 0.+0.0001, 1.0 ):
		res_fn = "muB_" + str(muB) + ".dat"
		f = open(res_fn,'w')
		f.write('T	muA	muB	cov	cov_A	cov_B	E''\n')
		for muA in np.arange(-20., 20.+0.0001, 2.0 ):
			res_cur = binary_gas_MC_simulation(muA, muB)
			res += res_cur
			for i in res_cur:
				f.write( ('{}\t'*7).format(*i) + '\n')
		f.close()
